const fs = require('fs-extra')
const path = require('path')
const archiver = require('archiver')
const tmp = require('tmp')
const request = require('request')
const ProgressBar = require('progress')
const loading = require('loading-indicator')

const endpoint = 'http://aco.aotu.io'
// const endpoint = 'http://localhost:8000'
const homePath = process.env[(process.platform === 'win32') ? 'USERPROFILE' : 'HOME']
const configFile = path.join(homePath, '.aco.json')

exports.loadConfig = function () {
  let config = null
  try {
    config = require(configFile)
  } catch (err) {
    console.log('  Load config error: ', err)
    config = {}
  }

  return config
}

exports.storeConfig = function (config) {
  fs.outputJSONSync(configFile, config)
}

exports.signup = function (username, password) {
  request.post({
    url: endpoint + '/api/signup',
    form: {username, password},
    json:true
  }, function (err, httpResponse, body) {
    if (body.code > 0) {
      console.log('  Signup failed: ', body.msg)
    } else {
      exports.storeConfig({token: body.data.token})
      console.log('  Signup ok')
    }
  })
}

exports.login = function (username, password) {
  request.post({
    url: endpoint + '/api/login',
    form: {username, password},
    json: true
  }, function (err, httpResponse, body) {
    if (body.code > 0) {
      console.log('  Login failed: ', body.msg)
    } else {
      exports.storeConfig({token: body.data.token})
      console.log('  Login ok')
    }
  })
}

exports.deploy = function deploy (token, dirPath, subdomain) {
  console.log(' ')
  const loadingTimer = loading.start('  Zip...')

  const tmpZipName = tmp.tmpNameSync({postfix: '.zip'})
  const tmpZip = fs.createWriteStream(tmpZipName)
  const archive = archiver('zip')

  // console.log('tmpZipName:', tmpZipName)

  archive.pipe(tmpZip)
  archive.directory(path.resolve(process.cwd(), dirPath), '.')
  archive.finalize()

  tmpZip.on('close', function () {
    loading.stop(loadingTimer)
    // console.log('stat', fs.statSync(tmpZipName))
    const zipSize = fs.statSync(tmpZipName).size
    const bar = new ProgressBar('  Uploading [:bar] :percent :etas', {
      complete: '=',
      incomplete: ' ',
      width: 20,
      total: zipSize
    })

    const formData = {
      token: token,
      files: fs.createReadStream(tmpZipName),
      subdomain: subdomain
    }
    const r = request.post({
      url: endpoint + '/api/page',
      formData: formData,
      json: true
    }, function (err, httpResponse, body) {
      // console.log('res body:', err, body)
      if (err) {
        return console.log('  Deploy error: ', err)
      }

      bar.update(1)
      if (body.code > 0 || !body.data || !body.data.host) {
        return console.log('  Deploy failed: ', body)
      }

      console.log('  Deploy successful!')
      console.log(' ')
      console.log('  Host: ', body.data.host)
      console.log(' ')
    }).on('drain', function () {
      // console.log('drain r.req.connection.bytesWritten:', r.req.connection.bytesWritten)
      let uploadedSize = r.req.connection.bytesWritten
      bar.update(uploadedSize/zipSize)
    })
  })
}


